#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Run with:  gpaw -P 4 python script_used.py
"""
Created on Sep 14 08:48:07 2021

@author: zhandos moldabekov (CASUS)
"""
import numpy as np
from ase import Atoms
from ase.units import Hartree, Bohr
#import math 
#from ase.io import write

from gpaw import GPAW, PW
from gpaw.xc import XC
from gpaw.external import ExternalPotential
from gpaw import Davidson
from gpaw.utilities.ps2ae import PS2AE
from ase.parallel import paropen 
from ase.parallel import parprint
from ase.build import niggli_reduce

from gpaw.xc.libxc import LibXC
from gpaw.xc.lda import LDA
from gpaw.xc.gga import GGA

import timeit

Ampl=0.1 #*Hartree,  the Amplitude of the external field


K_point=12  # the numebr of k-points
dl=0.15 ## in Angstrom, spacing  
Electron_number=14
N_band=280
Smearing_eV=12.528

max_en=440.000

def test_ext_potential_harmonic():
    a = 4.1  ## in Angstrom
    x = Atoms(cell=(a, a, a), pbc=True)  # no atoms
#    niggli_reduce(x)

    class HarmonicPotential(ExternalPotential):
        def calculate_potential(self, gd):
            r_vg = gd.get_grid_point_coordinates() # From Descriptor-class: The length unit is Bohr.
            ext_pot=2*Ampl* np.cos(2*np.pi*r_vg/(a/Bohr))
            self.vext_g =  ext_pot[2]  # use 2 for z, i.e. the perturbation is along z axis, for  Jellium x,y, and z directions are equivalent  

        def get_name(self):
            return 'HarmonicPotential'


    calc = GPAW(charge=-Electron_number,
                mode=PW(max_en),
                nbands=N_band,
                h=dl,
                xc=XC('LDA'),
                external=HarmonicPotential(),
                eigensolver='cg', 
                kpts={'size': (K_point, K_point, K_point), 'gamma': False}, # shifted K_pointxK_pointxK_point Monkhorst-pack
                occupations={'name': 'fermi-dirac',
                             'width': Smearing_eV},
                txt='cos.txt')


    x.calc = calc
    x.get_potential_energy()
    
    x.calc.hamiltonian.finegd.collect(calc.hamiltonian.vHt_g, broadcast=True)*Hartree
                                    
    gridrefinement = 1
    
    n=calc.get_all_electron_density(gridrefinement=gridrefinement,skip_core=False)

    nn=np.array(n)
    
    number_of_points=calc.get_number_of_grid_points()
    
    dv = x.get_volume() / calc.get_number_of_grid_points().prod()
    ddx=pow(dv,0.33333333333)
    
    I = np.sum(nn) * dv / gridrefinement**3
    Ix = nn[0,0]
    xxx=np.arange(number_of_points[0]*gridrefinement)*ddx/gridrefinement/a
    density_x=np.vstack((xxx, Ix)).T

    np.savetxt('density.dat',density_x)
    f = paropen('tot_den.dat', 'w')
    parprint('%5.2f' % (I), file=f)
    f.close()
    
    

test_ext_potential_harmonic()
